// Qt
#include <qlabel.h>
#include <qfile.h>
#include <qspinbox.h>
#include <qcombobox.h>
#include <qtextedit.h>
#include <qtimer.h>

#include <klistview.h>
#include <qlistview.h> 
#include <qwidgetstack.h> 

// KDE
#include <kmessagebox.h>
#include <kaboutapplication.h>
#include <kpopupmenu.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kactioncollection.h>
#include <kactionclasses.h>
#include <kapplication.h>
#include <kdialogbase.h>
#include <kconfig.h>
//#include <kglobal.h>
#include <kiconloader.h>
//#include <klocale.h>

//#include <kdebug.h>

// CPUInfo
#include "cpuinfo.h"
#include "flowlayout.h"
#include "sourcelistitem.h"
#include "prefs.h"

#include "sources/acpithermalsrc.h"
#include "sources/ibmacpithermalsrc.h"
#include "sources/ibmacpifansrc.h"
#include "sources/ibmhdaps.h"
#include "sources/omnibookthermalsrc.h"
#include "sources/ibookg4thermalsrc.h"
#include "sources/i2cthermalsrc.h"
#include "sources/sysfreqsrc.h"
#include "sources/cpuinfofreqsrc.h"

extern "C" {
	KPanelApplet* init( QWidget* inParent, const QString& inConfigFile) {
		KGlobal::locale()->insertCatalogue("cpuinfo");
		return new CPUInfo(inConfigFile, KPanelApplet::Normal,
		                      //KPanelApplet::About | KPanelApplet::Help | KPanelApplet::Preferences,
		                      KPanelApplet::About | KPanelApplet::Preferences,
		                      inParent, "cpuinfo");
	}
}

CPUInfo::CPUInfo(const QString& inConfigFile, Type inType, int inActions, QWidget* inParent, const char* inName)
		: KPanelApplet(inConfigFile, inType, inActions, inParent, inName),
		QToolTip(this),
		mCachedWFH(0),
		mCachedHeight(0){
	mAboutDialog = 0; // aboutData is created in the about() method
	mPrefsDlg = 0; // the prefs dialog is created in the preferences() method
	mPrefs = 0; // the prefs ui is created in the preferences() method

	// Get the current application configuration handle
	mKConfig = config();
	mKConfig->setGroup( "CPUInfo" );

	// create the sources
	mSources.setAutoDelete(TRUE);
	addSources(ACPIThermalSrc::createInstances(this));
	addSources(IBMACPIThermalSrc::createInstances(this));
	addSources(IBMACPIFanSrc::createInstances(this));
	addSources(IBMHDAPSSrc::createInstances(this));
	addSources(OmnibookThermalSrc::createInstances(this));
	addSources(IbookG4ThermalSrc::createInstances(this));
	addSources(I2CThermalSrc::createInstances(this));
	addSources(SysFreqSrc::createInstances(this));
	addSources(CpuinfoFreqSrc::createInstances(this));

	// create layout
	mLayout = new FlowLayout(this, orientation() );
	mLayout->setSpacing(8);

	// add the widgets to the layout
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it){
		if(source->isEnabled()){
			// enable transparency
			source->getWidget()->setBackgroundMode(X11ParentRelative);
			//source->getWidget()->setBackgroundOrigin(AncestorOrigin);
			mLayout->add(source->getWidget());
		}
	}

	// create the menu
	mMenu = new KPopupMenu( this );
	mMenu->insertTitle( SmallIcon( "hwinfo" ), i18n("CPU Info") );

	if (mCpufreqd.enabled()) {
		mMenu->insertItem(i18n("&Performance Profiles"), mCpufreqd.menu());
	}

	KActionCollection* actionCollection = new KActionCollection( this, "actionCollection", kapp );

	KAction* action = KStdAction::preferences( this, SLOT( preferences() ), actionCollection );
	action->setText( i18n("&Preferences") );
	action->plug( mMenu );

	action = KStdAction::aboutApp( this, SLOT( about() ), actionCollection );
	action->setText( i18n("&About CPU Info") );
	action->plug( mMenu );

	//menu->insertSeparator();
	//KStdAction::quit( kapp, SLOT( quit() ), actionCollection )->plug( menu );

	// connect the timer
	//connect( mRefreshTimer, SIGNAL(timeout()), this, SLOT(updateValues()) );

	//load prefs from the config
	loadPreferences();
}

CPUInfo::~CPUInfo() {
	//super class deletes the widgets
}

void CPUInfo::addSources(std::list<Source*> inList){
	for(std::list<Source*>::iterator it = inList.begin(); it != inList.end(); ++it){
		mSources.append(*it);
		connect(*it, SIGNAL(enabledChanged(bool, QWidget*)), this, SLOT(changeSource(bool, QWidget*))); // TODO: improve
	}
}

void CPUInfo::changeSource(bool inEnabled, QWidget* inWidget){
	if(inEnabled)
		mLayout->add(inWidget);
	else
		mLayout->remove(inWidget);
}

void CPUInfo::about() {
	if(!mAboutDialog){
		KAboutData* aboutData = new KAboutData("libcpuinfo", 
			I18N_NOOP("CPUInfo"),
			"0.5.1", 
			I18N_NOOP("Temperature/Frequency Applet.\n\n"
				"This applet shows the current CPU temperature and frequency.\n"
				"Make sure you have enabled a supported kernel module.\n"),
			KAboutData::License_GPL, "(c) 2004-2006, Ken Werner");
		aboutData->addAuthor("Ken Werner", 0, "ken.werner@web.de");
		aboutData->addAuthor("Valentine Sinitsyn", "cpufreqd control module", "e_val@inbox.ru");
		aboutData->addAuthor("Jocke Andersson", "source alignment improvement", "ajocke@gmail.com");
		aboutData->addAuthor("Flavio Castelli", "iBook G4 support", "micron@madlab.it");
		aboutData->addAuthor("Manfred Paul", "debian support", "manfredpaul@gmx.net");
		//aboutData->addCredit("blabla");
		mAboutDialog = new KAboutApplication(aboutData, this, "aboutApp", false);
	}
	mAboutDialog->show();
}

void CPUInfo::help() {
	KMessageBox::information(0, i18n("This is a help box"));
}

void CPUInfo::preferences() {
	//KMessageBox::information(0, i18n("This is a preferences box"));
	if(!mPrefsDlg){
		//create the prefs dialog
		mPrefsDlg = new KDialogBase( this, i18n("prefsDlg"), false, "Configuration", KDialogBase::Ok|KDialogBase::Apply|KDialogBase::Cancel, KDialogBase::Ok, false );
		//create the prefs ui
		mPrefs = new Prefs( mPrefsDlg, "prefsui" ); //the uic generated class

		//setup the dialog
		//mPrefsDlg->setFixedSize(mPrefs->size());
		mPrefsDlg->setMainWidget(mPrefs); //embed the uic generated class into the kde dialog
		connect(mPrefsDlg, SIGNAL(applyClicked()), this, SLOT(savePreferences()));
		connect(mPrefsDlg, SIGNAL(okClicked()), this, SLOT(savePreferences()));
		connect(mPrefsDlg, SIGNAL(cancelClicked()), this, SLOT(cancelPreferences()));

		// create new list items and the widgetStack widgets
		QPtrListIterator<Source> it(mSources);
		for(Source* source; (source = it.current()) != 0; ++it){
			// create the specific source prefs and add them to the widgetStack
			mPrefs->widgetStack->addWidget(source->createPrefs(mPrefs->widgetStack));
			// create the SourceListItem to switch and to enable/disable the widgetstack item (the source)
			SourceListItem* item = new SourceListItem(source, mPrefs->sourceListView, QCheckListItem::CheckBox);
			item->setOn(source->isEnabled());
		}

		if(mPrefs->sourceListView->firstChild()){
			mPrefs->sourceListView->setSelected(mPrefs->sourceListView->firstChild(), true);  // select the first list item
			raiseSourcePrefsWidget(mPrefs->sourceListView->firstChild()); // raise that item
		}

		connect(mPrefs->sourceListView, SIGNAL(selectionChanged(QListViewItem*)), this, SLOT(raiseSourcePrefsWidget(QListViewItem*)));
	}

	// update prefs gui
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it)
		source->updatePrefsGUI();
	//show the dialog
	mPrefsDlg->show();
}

void CPUInfo::raiseSourcePrefsWidget(QListViewItem* inListViewItem){
	mPrefs->widgetStack->raiseWidget(static_cast<SourceListItem*>(inListViewItem)->getSource()->getPrefs());
}

void CPUInfo::cancelPreferences(){
	// update prefs gui
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it)
		source->updatePrefsGUI();
}

void CPUInfo::savePreferences(){
	// apply prefs
	//QPtrListIterator<Source> it(mSources);
	//for(Source* source; (source = it.current()) != 0; ++it)
	//	source->applyPrefs();

	// save prefs (implicit apply)
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it)
		source->savePrefs(mKConfig);

	// write config
	mKConfig->sync();
	
	// reset cached witdh for height
	mCachedWFH = 0;
}

void CPUInfo::loadPreferences(){
	// load prefs
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it)
		source->loadPrefs(mKConfig);
}

int CPUInfo::widthForHeight(int inHeight) const {
	//kdDebug() << "widthForHeight: " << height << endl;
	mLayout->setOrientation(Qt::Horizontal);
	if(mCachedHeight != inHeight){
		mCachedHeight = inHeight;
		mCachedWFH = 0;
	}
 	mCachedWFH = QMAX(mLayout->widthForHeight(inHeight), mCachedWFH);
	return mCachedWFH;
}

int CPUInfo::heightForWidth(int inWidth) const {
	//kdDebug() << "heightForWidth: " << width << endl;
	mLayout->setOrientation(Qt::Vertical);
	return mLayout->heightForWidth(inWidth);
	//return sizeHint().height();
}

void CPUInfo::mousePressEvent( QMouseEvent* inEvent ) {
    if( inEvent->button() == QMouseEvent::RightButton )
        mMenu->popup( inEvent->globalPos() );
}

void CPUInfo::maybeTip( const QPoint& inPos ){
	if(!rect().contains(inPos))
		return;
	QString text = "<b>Sources:</b><br><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\">";
	QPtrListIterator<Source> it(mSources);
	for(Source* source; (source = it.current()) != 0; ++it){
		if(source->isToolTipEnabled())
			text.append("<tr><td>" + 
				source->getName() + "</td><td>" +
				source->getValue() + "</td><td>");
	}
	text.append("</table>");
    tip(rect(), text);
}
